#ifndef __BINTREE_H
#define __BINTREE_H

#include <assert.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

/**************************************
 *            DEFINITIONS
 *************************************/

#define MAX(X,Y) ((X) > (Y) ? (X) : (Y))

/*
 * Data type definition.
 */
typedef int data_t;

/*
 * Binary tree node definition.
 */
typedef struct bintreenode {
    data_t data;
    struct bintreenode *left;
    struct bintreenode *right;
} bintreenode_t;

/*
 * Binary tree definition.
 */
typedef struct {
    bintreenode_t *root;
} bintree_t;


/**************************************
 *         PROVIDED FUNCTIONS
 *************************************/

/*
 * Allocate a new binary tree node with the given data value.
 */
bintreenode_t* malloc_bintreenode (data_t data);

/*
 * Initialize an empty binary tree.
 */
void bintree_init (bintree_t *tree);

/*
 * Calculate the size of a tree.
 */
size_t bintree_size (bintree_t *tree);

/*
 * Calculate the height of a tree.
 */
size_t bintree_height (bintree_t *tree);


/**************************************
 *         TO BE IMPLEMENTED
 *************************************/

/*
 * Performs a preorder traversal of the tree, printing each value.
 */
void bintree_preorder (bintree_t *tree);

/*
 * Performs a postorder traversal of the tree, printing each value.
 */
void bintree_postorder (bintree_t *tree);

/*
 * Performs an inorder traversal of the tree, printing each value.
 */
void bintree_inorder (bintree_t *tree);

/*
 * Performs a breadth-first traversal of the tree, printing each value.
 */
void bintree_breadth_first (bintree_t *tree);

/*
 * Add the given value to a random leaf in the tree.
 */
void bintree_random_add (bintree_t *tree, data_t value);

/*
 * De-allocate all tree nodes.
 */
void bintree_free (bintree_t *tree);

#endif
