#include <assert.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

#include "turtle.h"
#include "bintree.h"
#include "drawtree.h"

/*
 * Create a 5-node hard-coded binary tree.
 */
void make_small_tree(bintree_t *tree)
{
    bintreenode_t *nodes[5];
    for (int i=0; i<5; i++) {
        nodes[i] = malloc_bintreenode(i);
    }
    nodes[1]->left = nodes[0];
    nodes[1]->right = nodes[2];
    nodes[3]->left = nodes[1];
    nodes[3]->right = nodes[4];
    tree->root = nodes[3];
}

/*
 * Create a 10-node hard-coded binary tree.
 */
void make_large_tree(bintree_t *tree)
{
    bintreenode_t *nodes[10];
    for (int i=0; i<10; i++) {
        nodes[i] = malloc_bintreenode(i);
    }
    nodes[1]->left  = nodes[0];
    nodes[1]->right = nodes[2];
    nodes[3]->left  = nodes[1];
    nodes[3]->right = nodes[4];
    nodes[4]->right = nodes[5];
    nodes[8]->left  = nodes[7];
    nodes[8]->right = nodes[9];
    nodes[6]->left  = nodes[3];
    nodes[6]->right = nodes[8];
    tree->root = nodes[6];
}

/*
 * Generate a random binary tree with the given size.
 */
void make_random_tree(bintree_t *tree, int size)
{
    for (int i=0; i<size; i++) {
        bintree_random_add(tree, i);
    }
}

int main()
{
    // initialize random number generator
    srand((unsigned)time(0));

    // initialize empty tree
    bintree_t my_tree;
    bintree_init(&my_tree);

    make_small_tree(&my_tree);          // 5-node hard-coded tree
    //make_large_tree(&my_tree);          // 10-node hard-coded tree
    //make_random_tree(&my_tree, 5);      // small random tree
    //make_random_tree(&my_tree, 50);     // medium random tree
    //make_random_tree(&my_tree, 100);    // large random tree

    // print tree info
    printf("Height:  %lu\n", bintree_height(&my_tree));
    printf("Size:    %lu\n", bintree_size(&my_tree));
    printf("Preorder:      ");
    bintree_preorder(&my_tree);
    printf("\nPostorder:     ");
    bintree_postorder(&my_tree);
    printf("\nInorder:       ");
    bintree_inorder(&my_tree);
    printf("\nBreadth-first: ");
    bintree_breadth_first(&my_tree);
    printf("\n");

    // output tree contents
    turtle_draw_bintree(&my_tree);

    // clean up
    bintree_free(&my_tree);
}
